@@ -32,6 +32,7 @@
 #include "layer/LayerFactory.h"
 #include "layer/FlexiNoteLayer.h"
 
+#include "widgets/MenuTitle.h"
 
 //!!! ugh
 #include "data/model/WaveFileModel.h"
@@ -46,6 +47,7 @@
 #include <QTextStream>
 #include <QMimeData>
 #include <QApplication>
+#include <QMenu>
 
 #include <iostream>
 #include <cmath>
@@ -54,6 +56,7 @@
 #include <QFrame>
 #include <QGridLayout>
 #include <QPushButton>
+
 #include "widgets/Thumbwheel.h"
 #include "widgets/Panner.h"
 #include "widgets/RangeInputDialog.h"
@@ -86,6 +89,7 @@ Pane::Pane(QWidget *w) :
     m_hthumb(nullptr),
     m_vthumb(nullptr),
     m_reset(nullptr),
+    m_lastVerticalPannerContextMenu(nullptr),
     m_mouseInWidget(false),
     m_playbackFrameMoveScheduled(false),
     m_playbackFrameMoveTo(0)
@@ -102,6 +106,11 @@ Pane::Pane(QWidget *w) :
     cerr << "Pane::Pane(" << this << ") returning" << endl;
 }
 
+Pane::~Pane()
+{
+    delete m_lastVerticalPannerContextMenu;
+}
+
 void
 Pane::updateHeadsUpDisplay()
 {
@@ -149,6 +158,11 @@ Pane::updateHeadsUpDisplay()
         connect(m_vpan, SIGNAL(mouseEntered()), this, SLOT(mouseEnteredWidget()));
         connect(m_vpan, SIGNAL(mouseLeft()), this, SLOT(mouseLeftWidget()));
 
+        // Panner doesn't provide its own context menu
+        m_vpan->setContextMenuPolicy(Qt::CustomContextMenu);
+        connect(m_vpan, SIGNAL(customContextMenuRequested(const QPoint &)),
+                this, SLOT(verticalPannerContextMenuRequested(const QPoint &)));
+
         m_vthumb = new Thumbwheel(Qt::Vertical);
         m_vthumb->setObjectName(tr("Vertical Zoom"));
         m_vthumb->setCursor(Qt::ArrowCursor);
@@ -2539,6 +2553,29 @@ Pane::verticalPannerMoved(float , float y0, float , float h)
     setTopLayerDisplayExtents(newmin, newmax);
 }
 
+void
+Pane::verticalPannerContextMenuRequested(const QPoint &pos)
+{
+    double vmin, vmax, dmin, dmax;
+    QString unit;
+    if (!getTopLayerDisplayExtents(vmin, vmax, dmin, dmax, &unit)) {
+        return;
+    }
+    
+    delete m_lastVerticalPannerContextMenu;
+    QMenu *m = new QMenu;
+    m_lastVerticalPannerContextMenu = m;
+
+    MenuTitle::addTitle(m, tr("Vertical Range: %1 - %2 %3")
+                        .arg(dmin).arg(dmax).arg(unit));
+
+    m->addAction(tr("&Edit..."), this, SLOT(editVerticalPannerExtents()));
+    m->addAction(tr("&Reset to Default"), this, SLOT(resetVerticalPannerExtents()));
+
+    m->popup(m_vpan->mapToGlobal(pos));
+    m_lastVerticalPannerContextMenu = m;
+}
+
 void
 Pane::editVerticalPannerExtents()
 {
@@ -2552,7 +2589,7 @@ Pane::editVerticalPannerExtents()
     }
 
     RangeInputDialog dialog(tr("Enter new range"),
-                            tr("New vertical display range, from %1 to %2 %4:")
+                            tr("New vertical display range, from %1 to %2 %3:")
                             .arg(vmin).arg(vmax).arg(unit),
                             unit, float(vmin), float(vmax), this);
     dialog.setRange(float(dmin), float(dmax));
@@ -2565,6 +2602,16 @@ Pane::editVerticalPannerExtents()
     }
 }
 
+void
+Pane::resetVerticalPannerExtents()
+{
+    if (m_vthumb) {
+        // This determines the "size" of the panner box
+        m_vthumb->resetToDefault();
+    }
+    m_vpan->resetToDefault();
+}
+
 void
 Pane::layerParametersChanged()
 {
