@@ -32,6 +32,7 @@
 #include <QPushButton>
 #include <QSplitter>
 #include <QStackedWidget>
+#include <QResizeEvent>
 
 #include <iostream>
 
@@ -146,6 +147,8 @@ PaneStack::addPane()
         properties = new PropertyStack(frame, pane);
         connect(properties, SIGNAL(propertyContainerSelected(View *, PropertyContainer *)),
                 this, SLOT(propertyContainerSelected(View *, PropertyContainer *)));
+        connect(properties, SIGNAL(propertyContainerContextMenuRequested(View *, PropertyContainer *, QPoint)),
+                this, SLOT(propertyContainerContextMenuRequested(View *, PropertyContainer *, QPoint)));
         connect(properties, SIGNAL(viewSelected(View  *)),
                 this, SLOT(viewSelected(View *)));
         connect(properties, SIGNAL(contextHelpChanged(const QString &)),
@@ -223,6 +226,8 @@ PaneStack::relinkAlignmentViews()
             av->show();
         }
     }
+
+    adjustAlignmentViewHeights(size().height());
 }
 
 void
@@ -236,6 +241,44 @@ PaneStack::unlinkAlignmentViews()
     }
 }
 
+void
+PaneStack::resizeEvent(QResizeEvent *ev)
+{
+    adjustAlignmentViewHeights(ev->size().height());
+}
+
+void
+PaneStack::adjustAlignmentViewHeights(int forMyHeight)
+{
+    if (!(m_options & int(Option::ShowAlignmentViews))) return;
+    if (!(m_options & int(Option::NoUserResize))) return;
+    if (!isVisible()) return;
+
+    int heightPerPane = forMyHeight / int(m_panes.size());
+
+    SVCERR << "heightPerPane = " << heightPerPane << " ("
+           << forMyHeight << "/" << m_panes.size() << ")" << endl;
+    
+    int roomForAlignmentView = heightPerPane / 4;
+    int min = ViewManager::scalePixelSize(6);
+    int max = ViewManager::scalePixelSize(25);
+    int alignmentHeight = roomForAlignmentView;
+    if (alignmentHeight < min) {
+        alignmentHeight = min;
+    }
+    if (alignmentHeight > max) {
+        alignmentHeight = max;
+    }
+
+    SVCERR << "alignmentHeight = " << alignmentHeight << endl;
+    
+    for (int i = 0; in_range_for(m_panes, i); ++i) {
+        auto av = m_panes[i].alignmentView;
+        if (!av) continue;
+        av->setFixedHeight(alignmentHeight);
+    }
+}
+
 void
 PaneStack::setPropertyStackMinWidth(int mw)
 {
@@ -587,6 +630,23 @@ PaneStack::propertyContainerSelected(View *client, PropertyContainer *pc)
     else emit currentLayerChanged(m_currentPane, nullptr);
 }
 
+void
+PaneStack::propertyContainerContextMenuRequested(View *client,
+                                                 PropertyContainer *pc,
+                                                 QPoint pos)
+{
+    Pane *pane = dynamic_cast<Pane *>(client);
+    Layer *layer = dynamic_cast<Layer *>(pc);
+
+    if (pane) {
+        if (layer) {
+            emit layerPropertiesRightButtonMenuRequested(pane, layer, pos);
+        } else {
+            emit panePropertiesRightButtonMenuRequested(pane, pos);
+        }
+    }
+}
+
 void
 PaneStack::viewSelected(View *v)
 {
@@ -607,7 +667,7 @@ PaneStack::rightButtonMenuRequested(QPoint position)
 {
     Pane *pane = dynamic_cast<Pane *>(sender());
     if (!pane) return;
-    emit rightButtonMenuRequested(pane, position);
+    emit paneRightButtonMenuRequested(pane, position);
 }
 
 void
